﻿using System;
using System.Windows.Forms;
using VTableNamespace;
using Word = Microsoft.Office.Interop.Word;

namespace TableStructurer
{
    /// <summary>
    /// Klasa do pobierania naglowka dokumentu Word, oraz wartosci i szerokosci komorek tego naglowka,
    /// a także tworzenia macierzy pozycjonujacej z wykorzystaniem metody SumOfWidths
    /// <remarks>
    /// Klasa wykorzystuje biblioteke COM: Microsoft Word 14.0 Object Library,
    /// ktora nalezy dodac do referencji projektu (w przypadku programow MS Word innych niz wersja 2010
    /// numer wersji biblioteki moze sie roznic)
    /// </remarks>
    /// </summary>
    class TableHeader
    {
        private const char Tab = (char)7;
        private const char Space = (char)32;

        /// <summary>Pole - aplikacja MS Word</summary>
        private static Word.Application word;

        /// <summary>Pole - dokument MS Word</summary>
        private static Word.Document doc;

        /// <summary>Pole - naglowek dokumentu</summary>
        private static VTable THeader;

        /// <summary>Pole - macierz pozycjonujaca</summary>
        private static VTable TMatrix;

        /// <summary>Pole - ilosc kolumn tabeli w naglowku</summary>
        private static int ColumnCount;

        /// <summary>Metoda pobierajaca naglowek dokumentu Word
        /// <remarks>
        /// Przyjeto ze naglowek to jednowierszowa tablica zawierajaca minimum trzy komorki
        /// gdyz dla mniejszej ilosci komorek nawet w przypadku scalen w tabelach
        /// nie ma potrzeby zmiany ich indeksow
        /// przy porzadkowaniu danych w celu dalszego przetwarzania
        /// </remarks>
        /// </summary>
        /// <param name="aWord">Aplikacja MS Word powiazana przez referencje COM</param>
        /// <param name="aDoc">Dokument MS Word otwierany przez aplikacje</param>
        /// <param name="tHeader">
        /// Tworzony przez metode obiekt klasy VTable zawierajacy
        /// wartosci i szerokosci komorek naglowka dokumetu Word
        /// </param>
        /// <param name="tMatrix">
        /// Tworzona przez metode macierz pozycjonujaca klasy VTable
        /// zawierajaca granice dla szerokosci komorek dokumentu tabeli Word
        /// pozwalajace na ustalenie, czy i o ile nalezy zmieniac indeksy komorek wierszy
        /// przy porzadkowniu danych
        /// </param>
        /// <param name="gridHeader">
        /// Opcjonalny komponent DataGridView prezentujacy naglowek dokumentu
        /// </param>
        /// <param name="gridMatrix">
        /// Opcjonalny komponent DataGridView prezentujacy maciez pozycjonujaca
        /// </param>
        /// <returns>True w przypadku powodzenia, False w przypadku niepowodzenia mentody</returns>
        public static bool GetHeader(
            Word.Application aWord,
            Word.Document aDoc,
            ref VTable tHeader,
            ref VTable tMatrix,
            DataGridView gridHeader = null,
            DataGridView gridMatrix = null
            )
        {
            word = aWord;
            doc = aDoc;
            THeader = tHeader;
            TMatrix = tMatrix;

            Word.HeaderFooter header = doc.Sections[1].Headers[Word.WdHeaderFooterIndex.wdHeaderFooterPrimary];
            
            if (header.Range.Tables.Count != 1)
            {
                MessageBox.Show("Nagłówek musi zawierać jedną tabelę");
                return false;
            }

            Word.Table headerTable = header.Range.Tables[1];
            
            ColumnCount = headerTable.Columns.Count;

            if (ColumnCount < 3)
            {
                MessageBox.Show("Tabela w nagłówku musi zawierać co najmniej 3 kolumny");
                return false;
            }

            if (headerTable.Rows.Count != 1)
            {
                MessageBox.Show("Tabela w nagłówku musi zawierać jeden wiersz");
                return false;
            }

            THeader = new VTable(ColumnCount, gridHeader);
            TMatrix = new VTable(ColumnCount, gridMatrix);
            THeader.RowCount = 1;
            TMatrix.RowCount = ColumnCount;

            for (int c = 1; c <= ColumnCount; c++)
            {
                THeader[c - 1, 0].AsStr = headerTable.Cell(1, c).Range.Text.Replace(Tab, Space).Trim(); ;
                THeader[c - 1, 0].AsInt = (int)Math.Round(headerTable.Cell(1, c).Width, 0);
            }

            for (int c = 0; c < THeader.ColumnCount; c++)
            {
                for (int count = c + 1; count < THeader.ColumnCount; count++)
                {
                    TMatrix[c, count - 1].AsInt = SumOfWidths(c, count);
                }
            }

            for (int c = 0; c < THeader.ColumnCount; c++)
            {
                int i1 = TMatrix[c, TMatrix.RowCount - 2].AsInt;
                int i2 = THeader[THeader.ColumnCount - 1, 0].AsInt;
  
                TMatrix[c, TMatrix.RowCount - 1].AsInt = i1 + i2;
            }
            
            TMatrix[TMatrix.ColumnCount - 1, TMatrix.RowCount - 1].AsInt 
                = 3 * THeader[THeader.ColumnCount - 1, 0].AsInt / 2;
            tHeader = THeader;
            tMatrix = TMatrix;
            return true;
        }

        /// <summary>Metoda uzywana przy wypelnianiu tMatrix danymi</summary>
        /// <param name="startIdx">Indeks poczatkowej komorki</param>
        /// <param name="count">Ilosc komorek scalonych</param>
        /// <returns>
        /// Suma szerokosci komorek tabeli Word dla indeksu startIdx oraz ilosci scalonych komorek count
        /// <remarks>
        /// Suma jest powiekszona o polowe szerokosci ostatniej komorki dla danego indeksu
        /// i danej ilosci scalonych komorek, a dla ostatniej komorki w wierszu tabeli Word
        /// jest to 3/2 jej szerokosci
        /// </remarks>
        ///</returns>
        private static int SumOfWidths(int startIdx, int count)
        {
            int sum = 0;
            
            for (int i = startIdx; i < count; i++)
            {
                sum += THeader[i, 0].AsInt;
            }
            
            sum += THeader[count, 0].AsInt / 2;
            return sum;
        }
    }
}